@@ -1,6 +1,6 @@
 package Markdent::Dialect::Theory::BlockParser;
 BEGIN {
-  $Markdent::Dialect::Theory::BlockParser::VERSION = '0.12';
+  $Markdent::Dialect::Theory::BlockParser::VERSION = '0.13';
 }
 
 use strict;
@@ -129,7 +129,7 @@ sub _match_table {
     ) if defined $caption && $self->debug();
 
     my $header = $3;
-    my $body = $4;
+    my $body   = $4;
 
     $self->_debug_parse_result(
         $header,
@@ -155,7 +155,7 @@ sub _match_table {
     if (@header) {
         my $first_header_cell_content = $header[0][0]{content};
         unless ( defined $first_header_cell_content
-                 && $first_header_cell_content =~ /\S/ ) {
+            && $first_header_cell_content =~ /\S/ ) {
             $_->[0]{is_header_cell} = 1 for @body;
         }
     }
@@ -167,7 +167,7 @@ sub _match_table {
 
     $self->_events_for_rows( \@header, 'Header' )
         if @header;
-    $self->_events_for_rows( \@body,   'Body' );
+    $self->_events_for_rows( \@body, 'Body' );
 
     $self->_send_event('EndTable');
 
@@ -177,9 +177,9 @@ sub _match_table {
 }
 
 sub _parse_rows {
-    my $self = shift;
+    my $self     = shift;
     my $split_re = shift;
-    my $rows = shift;
+    my $rows     = shift;
 
     my @rows;
 
@@ -202,7 +202,9 @@ sub _parse_rows {
                 for my $i ( 0 .. $#{$cells} ) {
                     if ( defined $cells->[$i]{content}
                         && $cells->[$i]{content} =~ /\S/ ) {
-                        $rows[-1][$i]{content} .= "\n" . $cells->[$i]{content};
+                        $rows[-1][$i]{content}
+                            .= "\n" . $cells->[$i]{content};
+                        $rows[-1][$i]{colspan} ||= 1;
                     }
                 }
             }
@@ -223,7 +225,7 @@ sub _is_continuation_line {
         if $line =~ /(?<!\\)[|]/x;
 
     return 1
-        if $line =~ /(?<!\\):/x;
+        if $line =~ /(^|\p{SpaceSeparator}+)(?<!\\):(\p{SpaceSeparator}|$)/x;
 
     # a blank line, presumably
     return 0;
@@ -236,11 +238,14 @@ sub _cells_from_line {
 
     my @row;
 
-    for my $cell ( $self->_split_cells($line, $div) ) {
+    for my $cell ( $self->_split_cells( $line, $div ) ) {
         if ( length $cell ) {
             push @row, $self->_cell_params($cell);
         }
-        # If the first cell is empty, we just treat it as an empty cell.
+        # If the first cell is empty, that means the line started with a
+        # divider, and we can ignore the "cell". If we already have cells in
+        # the row, that means we just saw a repeated divider, which means the
+        # most recent cell has a colspan+1.
         elsif (@row) {
             $row[-1]{colspan}++;
         }
@@ -255,7 +260,7 @@ sub _split_cells {
     my $div  = shift;
 
     $line =~ s/^\Q$div//;
-    $line =~ s/\Q$div\E\p{SpaceSeparator}*$/$div/;
+    $line =~ s/\Q$div\E$HorizontalWS*$/$div/;
 
     # We don't want to split on a backslash-escaped divider, thus the
     # lookbehind. The -1 ensures that Perl gives us the trailing empty fields.
@@ -281,7 +286,7 @@ sub _cell_params {
         $alignment = $self->_alignment_for_cell($cell);
 
         ( $content = $cell )
-            =~ s/^\p{SpaceSeparator}+|\p{SpaceSeparator}+$//g;
+            =~ s/^$HorizontalWS+|$HorizontalWS+$//g;
     }
 
     my %p = (
@@ -323,13 +328,14 @@ sub _normalize_cell_count_and_alignments {
     # "left". We loop through all the rules and set alignments accordingly.
     my %alignments;
 
-    for my $row ( grep { defined } @rows ) {
+    for my $row ( grep {defined} @rows ) {
+
         # If we have one extra column and the final cell has a colspan > 1 it
         # means we misinterpreted a trailing divider as indicating that the
         # prior cell had a colspan > 1. We adjust for that by comparing it to
         # the number of columns in the first row.
         if ( sum( map { $_->{colspan} } @{$row} ) == $default_cells + 1
-             && $row->[-1]{colspan} > 1 ) {
+            && $row->[-1]{colspan} > 1 ) {
             $row->[-1]{colspan}--;
         }
 
@@ -353,12 +359,12 @@ sub _events_for_rows {
     my $type = shift;
 
     my $start = 'StartTable' . $type;
-    my $end = 'EndTable' . $type;
+    my $end   = 'EndTable' . $type;
 
     $self->_send_event($start);
 
     for my $row ( @{$rows} ) {
-        if ( ! defined $row ) {
+        if ( !defined $row ) {
             $self->_send_event($end);
             $self->_send_event($start);
             next;
@@ -375,13 +381,14 @@ sub _events_for_rows {
             );
 
             if ( defined $content ) {
+
                 # If the content has newlines, it should be matched as a
                 # block-level construct (blockquote, list, etc), but to make
                 # that work, it has to have a trailing newline.
                 $content .= "\n"
                     if $content =~ /\n/;
 
-                $self->_parse_text(\$content);
+                $self->_parse_text( \$content );
             }
 
             $self->_send_event(
@@ -436,7 +443,7 @@ Markdent::Dialect::Theory::BlockParser - Block parser for Theory's Markdown
 
 =head1 VERSION
 
-version 0.12
+version 0.13
 
 =head1 DESCRIPTION
 
